/*
 * Original Copyright (C) 2013 Sylvain Munaut <tnt@246tNt.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 * THIS WAS MODDED <beyong recognition> for the FLIR Ex series Hack
 *
 */


#include <stdio.h>
#include "crmd160.h"
#include <iostream>
#include <string>
#include <sys/stat.h>
#include <sys/types.h>

void hexdump(void *ptr, int buflen) {
  unsigned char *buf = (unsigned char*)ptr;
  int i, j;
  for (i=0; i<buflen; i+=16) {
	if (buflen - i < 50) { // limit hexdump to footer
		printf("%06x: ", i);
		for (j=0; j<16; j++)
		  if (i+j < buflen)
			printf("%02x ", buf[i+j]);
		  else
			printf("   ");
		printf(" ");
		for (j=0; j<16; j++)
		  if (i+j < buflen)
			printf("%c", isprint(buf[i+j]) ? buf[i+j] : '.');
		printf("\n");
	}
  }
}


void *file_load(const char *filename, int *len)
{
	FILE *fh;
	void *data;

	fh = fopen(filename, "rb");
	if (!fh)
		return NULL;

	fseek(fh, 0, SEEK_END);
	*len = ftell(fh);
	fseek(fh, 0, SEEK_SET);

	data = malloc(*len);

	fread(data, 1, *len, fh);

	fclose(fh);

	return data;
}




int main(int argc, char *argv[])
{
	void *data_buf;
	char *mangled;
	int data_len;
	CRMD160 *hash;
	char digest[20];

	char * filename = "template.cfg";
	if (argc != 2) {
		fprintf(stdout, "Usage: %s infile.txt... defaulting to template.cfg\n", argv[0]);
	} else {
		filename = argv[1];
		printf("Using %s as filename\n",filename);
	}

	data_buf = file_load(filename, &data_len);
	if (!data_buf) {
		fprintf(stderr, "[!] Unable to open input file\n");
		return -1;
	}

	char * pch;
	char * searchfor = "# ID xxxxxxxx";

	pch = strstr ((char*)data_buf,searchfor);
	if (pch) {
		printf("BEFORE PATCHING:\n");
		hexdump(data_buf,data_len);

		printf("FOUND DUMMY ID-FIELD - PLEASE ENTER YOUR CAMERA'S SERIAL NUMBER:");
		std::string mySN;
		std::getline(std::cin,mySN);
		char * serial = (char *)malloc(mySN.size() + 1);
		memcpy(serial, mySN.c_str(), mySN.size() + 1);

		//sanity check SN
		if (strlen(serial) != 8) {
			printf ("Serial number has wrong length!");
			return -2;
		}

		char * replaceby = "# ID ";
		char str1[15];
		char str2[15];
		strcpy (str1,replaceby);
		strcpy (str2,serial);
		strncat (str1, str2, strlen(serial));
		replaceby = str1;


		printf ("Replacing '%s' with '%s'\n", searchfor, replaceby);
		strncpy (pch, replaceby, strlen(searchfor));
		printf("AFTER PATCHING:\n");
	} else {
		printf("WARNING: No replaceable string '%s' found!\n", searchfor);
		printf("         Already patched?\n");
	}

	hexdump(data_buf,data_len);

	int oldsize = data_len;
	char * crcpos;
	char * searchfor2 = "# CRC";
	crcpos = strstr ((char*)data_buf,searchfor2); //location of CRC code
	if (crcpos) {
		printf ("Found # CRC at offset %d \n", crcpos-data_buf);
		data_len = (crcpos-data_buf); //shorten length pointer = cutoff file-end
	} else {
		printf("ERROR: File contains no CRC\n");
		return -5;
	}
	int crcoffset = data_len;

	//compute hash
	hash = new CRMD160(1);
	hash->update((const char *)data_buf, data_len);
	hash->digest(digest);

	char crcstring[18];
	sprintf(crcstring, "# CRC01 %02x%02x%02x%02x\r\n",digest[3]&0xff, digest[2]&0xff, digest[1]&0xff, digest[0]&0xff);
	delete hash;
	printf("Computed CRC:\n");
	hexdump(crcstring,18);

	// now we have everything needed to re-assemble the file :)

	int totalsize=data_len + 18; //lenght of base + length of CRC-footer
	char * result = (char *)malloc(totalsize + 1);
    memcpy(result, 			data_buf, data_len);
    memcpy(result + crcoffset,	crcstring, 18);
    printf ("RESULT:\n");
    hexdump(result,totalsize);

    // last step: write file back to filesystem


    // DIRTY - we con't check if the path exists
    FILE * out = fopen (".\\ZIPMYCONTENTS\\flashfs\\system\\appcore.d\\config.d\\e8.cfg", "wb");
    fwrite (result, sizeof(char), totalsize, out);
    fclose (out);
    printf ("Have fun with your new .\\ZIPMYCONTENTS\\flashfs\\system\\appcore.d\\config.d\\e8.cfg !\n\n");
    printf ("\n\nNEXT STEPS:\n\n zip the contents of ZIPMYCONTENTS (without that directory itself)\n");
    printf ("... then rename the zip-file from .zip to .fif\n");
    printf ("Use FLIRInstallNet from your FlirTools installation to install this file\n");
    printf ("Press enter to close this window\n");
	std::string dummy;
	std::getline(std::cin,dummy);


	free(data_buf);
	free(result);
	//heh, why bother with cleaning up... ;)
	return 0;
}




